@@ -1,5 +1,3 @@
-use std::ffi::{OsStr,OsString};
-use std::os::unix::ffi::OsStrExt;
 use std::error::Error;
 use std::boxed::Box;
 use std::fs::{read,read_dir};
@@ -15,11 +13,7 @@ use nix::unistd::{sysconf,SysconfVar};
 use nix::time::{clock_gettime,ClockId};
 use nix::sys::time::TimeSpec;
 
-use serde::{Serialize,Serializer};
-use serde::ser::{SerializeStruct};
-
 use crate::types::{EventID,Record,Value,Number};
-use crate::quoted_string::ToQuotedString;
 
 lazy_static! {
     /// kernel clock ticks per second
@@ -34,23 +28,7 @@ pub struct Process {
     /// parent process id
     pub ppid: u64,
     /// command line
-    pub argv: Vec<OsString>
-}
-
-impl Serialize for Process {
-    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
-    where S: Serializer
-    {
-        let mut s = serializer.serialize_struct("Process", 3)?;
-        let launch_time = self.launch_time as f64 / 1000.0;
-        s.serialize_field("ARGV", &self.argv
-                          .iter()
-                          .map(|s|s.as_bytes().to_quoted_string())
-                          .collect::<Vec<_>>())?;
-        s.serialize_field("launch_time", &launch_time)?;
-        s.serialize_field("ppid", &self.ppid)?;
-        s.end()
-    }
+    pub argv: Vec<Vec<u8>>
 }
 
 impl Process {
@@ -60,8 +38,8 @@ impl Process {
         let argv = read(format!("/proc/{}/cmdline", pid))
             .map_err(|e| format!("read /proc/{}/cmdline: {}", pid, e))?
             .split(|c| *c == 0)
-            .filter(|s|!s.is_empty())
-            .map(|s|OsStr::from_bytes(s).to_os_string())
+            .filter(|s| !s.is_empty())
+            .map(|s| s.to_owned())
             .collect::<Vec<_>>();
 
         let buf = read(format!("/proc/{}/stat", pid))
@@ -117,14 +95,12 @@ impl Process {
         } else {
             return Err("ppid field not found".into());
         }
-        match rexecve.get(b"ARGV") {
-            Some(v) =>  {
-                let l: Vec<Vec<u8>> = v.try_into()?;
-                p.argv = l.iter()
-                    .map(|elem| OsStr::from_bytes(&elem).into() )
-                    .collect::<Vec<OsString>>();
-            },
-            _ => return Err("ARGV field not found".into()),
+        if let Some(v) = rexecve.get(b"ARGV") {
+            p.argv = v.try_into()?;
+        } else if let Some(v) = rexecve.get(b"ARGV_STR") {
+            p.argv = v.try_into()?;
+        } else {
+            return Err("ARGV field not found".into());
         }
         Ok((pid, p))
     }
@@ -218,6 +194,16 @@ impl ProcTable {
     }
 }
 
+/// Returns environment for a given process
+pub fn get_environ(pid: u64) -> Result<Vec<(Vec<u8>,Vec<u8>)>, Box<dyn Error>> {
+    Ok(read(format!("/proc/{}/environ", pid))?
+       .split(|c| *c == 0)
+       .map(|f| {
+           let mut kv = f.splitn(2, |c| *c == b'=');
+           (kv.next().unwrap().to_owned(), kv.next().or(Some(b"")).unwrap().to_owned())
+       }).collect())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
